import org.apache.poi.ss.usermodel.*;
import java.io.*;

// Demonstrates the setColumnHidden bug.

// This app reads in a test workbook having a sheet with nine columns
// (A-I) and a couple of rows, one to number the columns and one to
// show how wide Excel thinks each column is. The app adds a third row
// to show the column width according to POI XSSF.

// Then it uses the setColumnHidden method to hide columns C and G,
// after which it writes out the modified workbook.

// It succeeds in hiding columns C and G. However, the bug is that
// columns B and D are also hidden. Apparently, this is because B and
// D are adjacent to C and have the same width as C, whereas column G's
// neighbors have widths different from that of G.

class HideColumnTest
{
  public static void main(String[] args)
  {
    try {
      new HideColumnTest().run();
    }
    catch(Throwable e) {
      e.printStackTrace();
    }
  }
  
  void run() throws Exception
  {
    FileInputStream fis = new FileInputStream("input.xlsx");
    Workbook wb = WorkbookFactory.create(fis);
    Sheet sheet = wb.getSheetAt(0);
    
    Row row1 = sheet.getRow(1);
    Row row2 = sheet.createRow(2);
    DataFormat df = wb.createDataFormat();
    CellStyle style = wb.createCellStyle();
    style.setDataFormat(df.getFormat("0.000"));
    for (int colNum=0; colNum<row1.getLastCellNum(); ++colNum) {
      Cell cell = row2.createCell(colNum);
      cell.setCellValue(sheet.getColumnWidth(colNum)/256.0);
      cell.setCellStyle(style);
    }
    sheet.setColumnHidden(2, true); // Column C
    sheet.setColumnHidden(6, true); // Column G
    
    FileOutputStream out = new FileOutputStream("output.xlsx");
    wb.write(out);
    out.close();
  }
}